@@ -17,13 +17,14 @@ module Hledger.UI.Theme (
 )
 where
 
-import qualified Data.Map as M
+import Data.Map qualified as M
 import Data.Maybe
 import Graphics.Vty
 import Brick
+import Safe (headErr)
 
 defaultTheme :: AttrMap
-defaultTheme = fromMaybe (snd $ head themesList) $ getTheme "white"
+defaultTheme = fromMaybe (snd $ headErr themesList) $ getTheme "white"  -- PARTIAL headErr succeeds because themesList is non-null
   -- the theme named here should exist;
   -- otherwise it will take the first one from the list,
   -- which must be non-empty.
@@ -103,6 +104,29 @@ themesList = [
     (attrName "list" <> attrName "selected"                             , defAttr & reverseVideo)
   ])
 
+  ,("dark", attrMap (white `on` black & dim) [
+      (attrName "border"                                                                   , white `on` black)
+    , (attrName "border" <> attrName "bold"                                                , currentAttr & bold)
+    , (attrName "border" <> attrName "depth"                                               , active)
+    , (attrName "border" <> attrName "filename"                                            , currentAttr)
+    , (attrName "border" <> attrName "key"                                                 , active)
+    , (attrName "border" <> attrName "minibuffer"                                          , white `on` black & bold)
+    , (attrName "border" <> attrName "query"                                               , active)
+    , (attrName "border" <> attrName "selected"                                            , active)
+    , (attrName "error"                                                                    , fg red)
+    , (attrName "help"                                                                     , currentAttr & bold)
+    , (attrName "help" <> attrName "heading"                                               , fg blue)
+    , (attrName "help" <> attrName "key"                                                   , active)
+    , (attrName "list" <> attrName "amount" <> attrName "decrease"                         , fg red)
+    , (attrName "list" <> attrName "amount" <> attrName "decrease" <> attrName "selected"  , red `on` black & bold)
+    , (attrName "list" <> attrName "balance"                                               , currentAttr)
+    , (attrName "list" <> attrName "balance" <> attrName "negative"                        , fg red)
+    , (attrName "list" <> attrName "balance" <> attrName "positive"                        , fg white)
+    , (attrName "list" <> attrName "balance" <> attrName "negative" <> attrName "selected" , red `on` black    & bold)
+    , (attrName "list" <> attrName "balance" <> attrName "positive" <> attrName "selected" , yellow `on` black & bold)
+    , (attrName "list" <> attrName "selected"                                              , yellow `on` black & bold)
+  ])
+
   ]
 
 -- halfbrightattr = defAttr & dim
