@@ -12,14 +12,15 @@ module Hledger.UI.TransactionScreen
 ,tsHandle
 ) where
 
-import Control.Monad
+import Brick
+import Brick.Widgets.Edit (editorText, renderEditor)
+import Brick.Widgets.List (listMoveTo)
 import Control.Monad.IO.Class (liftIO)
 import Data.List
 import Data.Maybe
-import qualified Data.Text as T
+import Data.Text qualified as T
 import Graphics.Vty (Event(..),Key(..),Modifier(..), Button (BLeft))
-import Brick
-import Brick.Widgets.List (listMoveTo)
+import System.Exit (ExitCode (..))
 
 import Hledger
 import Hledger.Cli hiding (mode, prices, progname,prognameandversion)
@@ -29,8 +30,8 @@ import Hledger.UI.UIState
 import Hledger.UI.UIUtils
 import Hledger.UI.UIScreens
 import Hledger.UI.Editor
-import Brick.Widgets.Edit (editorText, renderEditor)
-import Hledger.UI.ErrorScreen (uiReloadJournalIfChanged, uiCheckBalanceAssertions, uiReloadJournal)
+import Hledger.UI.ErrorScreen (uiCheckBalanceAssertions, uiReload, uiReloadIfFileChanged, uiToggleBalanceAssertions)
+import Hledger.UI.RegisterScreen (rsHandle)
 
 tsDraw :: UIState -> [Widget Name]
 tsDraw UIState{aopts=UIOpts{uoCliOpts=copts@CliOpts{reportspec_=rspec@ReportSpec{_rsReportOpts=ropts}}}
@@ -95,7 +96,7 @@ tsDraw UIState{aopts=UIOpts{uoCliOpts=copts@CliOpts{reportspec_=rspec@ReportSpec
               -- ,("q", "quit")
               ]
 
-tsDraw _ = errorWrongScreenType "draw function"  -- PARTIAL:
+tsDraw _ = errorWrongScreenType "tsDraw"  -- PARTIAL:
 
 -- Render a transaction suitably for the transaction screen.
 showTxn :: ReportOpts -> ReportSpec -> Journal -> Transaction -> T.Text
@@ -137,24 +138,31 @@ tsHandle ev = do
             VtyEvent (EvKey (KChar 'q') []) -> halt
             VtyEvent (EvKey KEsc        []) -> put' $ resetScreens d ui
             VtyEvent (EvKey (KChar c)   []) | c == '?' -> put' $ setMode Help ui
-            VtyEvent (EvKey (KChar 'E') []) -> suspendAndResume $ void (runEditor pos f) >> uiReloadJournalIfChanged copts d j ui
-              where
-                (pos,f) = case tsourcepos t of
-                            (SourcePos f' l1 c1,_) -> (Just (unPos l1, Just $ unPos c1),f')
+
+            -- g or file change: reload the journal and rebuild app state.
+            e | e `elem` [VtyEvent (EvKey (KChar 'g') []), AppEvent FileChange] ->
+              tsReload copts d ui
+
+              -- for debugging; leaving these here because they were hard to find
+              -- \u -> dbguiEv (pshow u) >> put' u  -- doesn't log
+              -- \UIState{aScreen=TS tss} -> error' $ pshow $ _tssTransaction tss
+
+            -- E: run editor, reload the journal.
+            VtyEvent (EvKey (KChar 'E') []) -> do
+              suspendAndResume' $ do
+                let (pos,f) = case tsourcepos t of (SourcePos f' l1 c1,_) -> (Just (unPos l1, Just $ unPos c1),f')
+                exitcode <- runEditor pos f
+                case exitcode of
+                  ExitSuccess   -> return ()
+                  ExitFailure c -> error' $ "running the text editor failed with exit code " ++ show c
+              tsReloadIfFileChanged copts d j ui
+
             AppEvent (DateChange old _) | isStandardPeriod p && p `periodContainsDate` old ->
               put' $ regenerateScreens j d $ setReportPeriod (DayPeriod d) ui
               where
                 p = reportPeriod ui
 
-            -- Reload. Warning, this updates parent screens but not the transaction screen itself (see tsUpdate).
-            -- To see the updated transaction, one must exit and re-enter the transaction screen.
-            e | e `elem` [VtyEvent (EvKey (KChar 'g') []), AppEvent FileChange] ->
-              liftIO (uiReloadJournal copts d ui) >>= put'
-                -- debugging.. leaving these here because they were hard to find
-                -- \u -> dbguiEv (pshow u) >> put' u  -- doesn't log
-                -- \UIState{aScreen=TS tss} -> error $ pshow $ _tssTransaction tss
-
-            VtyEvent (EvKey (KChar 'I') []) -> put' $ uiCheckBalanceAssertions d (toggleIgnoreBalanceAssertions ui)
+            VtyEvent (EvKey (KChar 'I') []) -> uiToggleBalanceAssertions d ui
 
             -- for toggles that may change the current/prev/next transactions,
             -- we must regenerate the transaction list, like the g handler above ? with regenerateTransactions ? TODO WIP
@@ -176,7 +184,54 @@ tsHandle ev = do
             VtyEvent (EvKey (KChar 'z') [MCtrl]) -> suspend ui
             _ -> return ()
 
-    _ -> errorWrongScreenType "event handler"
+    _ -> errorWrongScreenType "tsHandle"
+
+    where
+      -- Reload and fully regenerate the transaction screen.
+      -- XXX On transaction screen or below, this is tricky because of a current limitation of regenerateScreens.
+      -- For now we try to work around by re-entering the screen(s).
+      -- This can show flicker in the UI and it's hard to handle all situations robustly.
+      tsReload copts d ui = uiReload copts d ui >>= reEnterTransactionScreen copts d
+      tsReloadIfFileChanged copts d j ui = liftIO (uiReloadIfFileChanged copts d j ui) >>= reEnterTransactionScreen copts d
+      
+      reEnterTransactionScreen _copts d ui = do
+        -- 1. If uiReload (or checking balance assertions) moved us to the error screen, save that, and return to the transaction screen.
+        let
+          (merrscr, uiTxn) = case aScreen $ uiCheckBalanceAssertions d ui of
+            s@(ES _) -> (Just s,  popScreen ui)
+            _        -> (Nothing, ui)
+        -- 2. Exit to register screen
+        let uiReg = popScreen uiTxn
+        put' uiReg
+        -- 3. Re-enter the transaction screen
+        rsHandle (VtyEvent (EvKey KEnter [])) -- PARTIAL assumes we are on the register screen.
+        -- 4. Return to the error screen (below the transaction screen) if there was one.
+        -- Next events will be handled by esHandle. Error repair will return to the transaction screen.
+        maybe (return ()) (put' . flip pushScreen uiTxn) merrscr
+          -- doesn't uiTxn have old state from before step 3 ? seems to work
+
+        -- XXX some problem:
+        -- 4. Reload once more, possibly re-entering the error screen, by sending a g event.
+        -- sendVtyEvents [EvKey (KChar 'g') []]  --  XXX Might be disrupted if other events are queued
+
+        -- XXX doesn't update on non-error change:
+        -- 4. Reload once more, possibly re-entering the error screen.
+        -- uiTxnOrErr <- uiReload copts d uiTxn
+          -- uiReloadIfChanged ?
+          -- uiCheckBalanceAssertions ? seems unneeded
+        -- put' uiTxnOrErr
+
+        -- XXX not working right:
+        -- -- 1. If uiReload (or checking balance assertions) moved us to the error screen, exit to the transaction screen.
+        -- let
+        --   uiTxn = case aScreen $ uiCheckBalanceAssertions d ui of
+        --     ES _ -> popScreen ui
+        --     _    -> ui
+        -- -- 2. Exit to register screen
+        -- put' $ popScreen uiTxn
+        -- -- 3. Re-enter the transaction screen, and reload once more.
+        -- sendVtyEvents [EvKey KEnter [], EvKey (KChar 'g') []]  -- XXX Might be disrupted if other events are queued
+
 
 -- | Select a new transaction and update the previous register screen
 tsSelect :: Integer -> Transaction -> UIState -> UIState
